<?xml version="1.0"?>

<st-source>
<!-- Changes in change set PV -->
<time-stamp>From VisualWorks®, 7.3.1 of April 20, 2005 on August 20, 2008 at 6:25:18 pm</time-stamp>


<class>
<name>PvSAMJmsInterface</name>
<environment>PROVISO</environment>
<super>PROVISO.PvObject</super>
<private>false</private>
<indexed-type>none</indexed-type>
<inst-vars>javaPath jarPath jmsTimeout uri stdioConnection stdin stdout stderr eventStream externalProcess monitorService lastKeepAlive eventCallback stdoutListener stderrListener jmsClasses jmsProperties receivedKeepAlive user password queueId samVersion filter jmsEvents connection environment </inst-vars>
<class-inst-vars></class-inst-vars>
<imports></imports>
<category>SAM-Interface</category>
<attributes>
<package>SAM-Interface</package>
</attributes>
</class>

<methods>
<class-id>PROVISO.PvSAMJmsInterface</class-id> <category>process management</category>

<body package="SAM-Interface" selector="processStderr:">processStderr: testBlock 
	"This method is called by the eventService and is designed to run forever, reading infomration from the pipe connection with the external JMS listener process"
	
	self 
		processStrings: [self getStringFrom: stderr]
		do: [:str | self processErr: str]
		while: [testBlock value and: [externalProcess notNil]]</body>

<body package="SAM-Interface" selector="processStdout:">processStdout: testBlock 
	"This method is called by the eventService and is designed to run forever, reading infomration from the pipe connection with the external JMS listener process"
	
	self 
		processStrings: [self getStringFrom: stdout]
		do: [:str | self process: str]
		while: [testBlock value and: [externalProcess notNil]]</body>

<body package="SAM-Interface" selector="processStrings:do:while:">processStrings: stringCallback do: processCallback while: testBlock 
	"This method is called by the eventService and is designed to run forever, reading infomration from the pipe connection with the external JMS listener process"
	
	| str |
	[testBlock value] 
		whileTrue: 
			[[str := stringCallback value] 
				timeoutSeconds: 1
				onTimeout: [str := nil].
			str ifNotNil: [Process unsafeRegion: [processCallback value: str]]]</body>

<body package="SAM-Interface" selector="startExternalProcess">startExternalProcess
	
	| args |
	args := 
		Array 
			with: '-jar'
			with: jarPath
			with: uri host , ':' , uri port asString.
	externalProcess ifNotNil: [self stopExternalProcess].
	receivedKeepAlive := false.
	stdioConnection := 
			ExternalProcess defaultClass 
				pipeConnectionWithErrorFor: javaPath
				arguments: args
				setProcessDescriptor: [:p | externalProcess := p].
	self logProcessStarted: externalProcess pid.
	stdin := stdioConnection writeStream.
	stdout := stdioConnection readStream.
	stderr := stdioConnection errorReadStream.
	lastKeepAlive := Time microsecondClock.
	stderrListener restart.
	stdoutListener restart.
	"Only do the following if we talking to the 2.1 jms interface."
	self samVersion = '2.1' 
		ifTrue: 
			[self cmdAddUser: self user.
			self cmdAddPassword: (self hashedPassword).
			self cmdAddQueueId: self queueId].
	self samVersion = '3.0' 
		ifTrue: [self cmdAddUser: self user.
			    self cmdAddPassword: self password pvDecrypt.
			    self cmdAddQueueId: self queueId.
			    self cmdAddFilter: self filter]
		ifFalse: 
			[self jmsClasses do: [:ea | self cmdAddClass: ea].
			self jmsProperties do: [:ea | self cmdAddProperty: ea]].</body>

<body package="SAM-Interface" selector="stopExternalProcess">stopExternalProcess
	
	externalProcess ifNotNil: 
		[:process | 
		process isActive 
			ifTrue: 
				[self cmdQuit.
				(Delay forSeconds: 1) wait].
		self waitForExternalProcessToStop: self basicStopExternalProcess].
	stderrListener stop.
	stdoutListener stop.
	^self</body>
</methods>

<methods>
<class-id>PROVISO.PvSAMJmsInterface</class-id> <category>starting and stopping</category>

<body package="SAM-Interface" selector="start">start
	
	self uri ifNil: [^self].
	ObjectMemory addDependent: self.
	monitorService restart</body>

<body package="SAM-Interface" selector="stop">stop

	monitorService stop.
	ObjectMemory removeDependent: self.</body>
</methods>

<methods>
<class-id>PROVISO.PvSAMJmsInterface</class-id> <category>initialize-release</category>

<body package="SAM-Interface" selector="initialize">initialize

	super initialize.
	eventCallback := [ :event | ].
	lastKeepAlive := Time microsecondClock.
	receivedKeepAlive := false.
	javaPath := 'java'.
	jarPath := '/export/home/cmedev/spair/SAMJMS.jar'.
	uri := 'http://192.168.68.101:1099' asURI.
	jmsClasses := #().
	jmsProperties := #().
      jmsEvents := Dictionary new.
	jmsTimeout := 120.  "2 minutes"
      filter := ''.
	samVersion := '2.0'.
	monitorService :=
		Kom.PluggableService new
			name: 'SAM JMS Process Monitor';
			runCallback: [ :testBlock |  self reportDeathOf: [self runJmsProcess: testBlock] as: 'JMS Process Monitor'];
			priority: 45;
			yourself.
	stdoutListener :=
		Kom.PluggableService new
			name: 'SAM JMS stdout listener';
			runCallback: [ :testBlock | self reportDeathOf: [self processStdout: testBlock] as: 'JMS stdout listener'];
			priority: 45;
			yourself.
	stderrListener :=
		Kom.PluggableService new
			name: 'SAM JMS stderr listener';
			runCallback: [ :testBlock |  self reportDeathOf: [self processStderr: testBlock] as: 'JMS stderr listener'];
			priority: 45;
			yourself.</body>
</methods>


<methods>
<class-id>PROVISO.PvSAMInterface</class-id> <category>enumerating</category>

<body package="SAM-Interface" selector="peformFullResync">peformFullResync
	
	self attemptResync 
		ifTrue: [self critical: [self privateResync]]</body>
</methods>



</st-source>
